const axios = require('axios');
const fs = require('fs');
const path = require('path');
const updaterProdutosModel = require('../model/updaterProdutosModel');
const updaterProdVersaoModel = require('../model/updaterProdVersaoModel');
const updaterProdFiliModel = require('../model/updaterProdFiliModel');
const adqGetDashboardFiliais = require('../model/adqGetDashboardFiliais');
const updaterModel = require('../model/updaterModel');
const paginacao = require('../functions/Paginacao');
const OracleService = require('../service/OracleService');
const SystemLogger = require('../logs/SystemLogger');
const AssistantControl = require('../service/AssistantControl');
const Gzip = require('../functions/Gzip');
const Base64 = require('../functions/Base64');

class ApiController {
  async insertUpdaterProdutos(product) {
    const params = {
      ID:          product.ID,
      NMPRODUCT:   product.NMPRODUTO,
      IDUTMULTVER: product.IDUTASSMULTVER,
    };
    return await updaterProdutosModel.insertUpdaterProdutos(params);
  }

  async updateUpdaterProdutos(id, product) {
    const params = {
      ID:          id,
      NMPRODUCT:   product.NMPRODUTO,
      IDUTMULTVER: product.IDUTASSMULTVER,
    };
    return await updaterProdutosModel.updateUpdaterProdutos(params);
  }

  async insertUpdaterProdVersao(versions, updaterProdId, actualVersionIds) {
    await Promise.all(versions.map(async (version) => {
      const fullVersion = `${version.VERSION}.${version.REVISION}.${version.BUILD}`;
      if(!actualVersionIds || !actualVersionIds.includes(fullVersion)) {
        const params = {
          UPDATERPRODID: updaterProdId,
          NRVERSAO:      fullVersion,
          DSURL:         version.URL,
          DSDIRETORIO:   version.PATH_TO_DEPLOY,
        };
        await updaterProdVersaoModel.insertUpdaterProdVersao(params);
      }
    }));
  }

  async insertUpdaterProdFili(units, updaterProdId, actualUnitIds) {
    await Promise.all(units.map(async (unit) => {
      if(!actualUnitIds || !actualUnitIds.includes(unit)) {
        const params = {
          UPDATERPRODID: updaterProdId,
          CDFILIAL:      unit
        };
        await updaterProdFiliModel.insertUpdaterProdFili(params);
      }
    }));
  }

  async downloadVersion(url, fullVersion, productPath, id) {
    const { PACKAGE_FOLDER } = process.env;
    const productVersionPath = `${PACKAGE_FOLDER}/${productPath}`;
    const versionPath = `${productVersionPath}/${fullVersion}.tar.gz`;
    let writer = null;

    const connection = await OracleService.connect();

    try {
      if(!fs.existsSync(versionPath)) {
        if(!fs.existsSync(productVersionPath)) {
          fs.mkdirSync(productVersionPath, { recursive: true });
        }

        const writerPath = path.resolve(versionPath);
        writer = fs.createWriteStream(writerPath);
        
        const response = await axios({ url, method: 'GET', responseType: 'stream', });
        response.data.pipe(writer);  

        writer.on('finish', async () => {
          await this.updateUpdaterProdVersao(id, 'S');
          connection.commit();
        });
        
        writer.on('error', async (error) => {
          await this.updateUpdaterProdVersao(id, 'E');
          connection.commit();
          SystemLogger.error(`ERRO CREATE VERSION FILE ${error.message}`);
        });
      }
    } catch (error) {
      if (writer) {
        writer.close();
      }

      connection.commit();
      await this.updateUpdaterProdVersao(id, 'E');
      fs.unlinkSync(versionPath);
      SystemLogger.error(`ERRO DOWNLOAD VERSION ${error.message} URL: ${url}`);
    }
  }

  async updateUpdaterProdVersao(id, status) {
    const params = {
      ID: id,
      IDDOWNLOADVER: status,
    };

    await updaterProdVersaoModel.updateUpdaterProdVersaoStatus(params);
  }

  getFilesFromPath(directoryPath) {
    if (!fs.existsSync(directoryPath)) {
      return [];
    }

    const files = fs.readdirSync(directoryPath);
    const filtredFiles = [];
    files.forEach((file) => {
      const filePath = path.join(directoryPath, file);
      const stat = fs.statSync(filePath);

      if (!stat.isDirectory() && file.endsWith('.tar.gz')) {
        // Recursively call listTarGzFiles for subdirectories
        filtredFiles.push(file);
      }
    });

    return filtredFiles;
  }

  loadLocalPackages(localPackages, DSDIRETORIO) {
    const { PACKAGE_FOLDER } = process.env;
    localPackages[DSDIRETORIO] = localPackages[DSDIRETORIO] || [];
    localPackages[DSDIRETORIO] = localPackages[DSDIRETORIO].concat(
      this.getFilesFromPath(path.join(PACKAGE_FOLDER, DSDIRETORIO))
    );
  }

  async handleProductConfig() {
    const productsIdWithConfig = await updaterModel.getProductsIdWithConfig();
    const productsConfig = await AssistantControl.getProductsConfig();
    const mappedProductsConfig = productsConfig.reduce((acc, cv) => {
      acc[cv.devopsId] = cv.config;
      return acc;
    }, {});
    productsIdWithConfig.rows.forEach(({ CDPRODUCTID }) => {
      delete mappedProductsConfig[CDPRODUCTID];
    });
    const productsWithoutConfig = Object.keys(mappedProductsConfig);
    await Promise.all(productsWithoutConfig.map(CDPRODUCTID => {
      const decodedConfig = Gzip.decode(mappedProductsConfig[CDPRODUCTID]);
      return updaterModel.insertUpdaterProdConfig({
        CDPRODUCTID,
        NRVERSAO: '0.0.0',
        CONFIG: Base64.encode(decodedConfig),
      });
    }));
  }

  async setLatestUpdaterApprovedVersion() {
    try {
      const controlUrl = AssistantControl.getControlUrl();
      const credentials = AssistantControl.getCredentials();

      const response = await axios.get(`${controlUrl}/updater-approved-version`, { auth: credentials }, { timeout: 15000 });

      process.env.UPDATE_AVAILABLE_VERSION = response.data.version || 'error';
    } catch (error) {
      SystemLogger.error(`ERRO BUSCAR VERSAO DISPONIVEL UPDATER: ${error.message}`);
      process.env.UPDATE_AVAILABLE_VERSION = 'error';
    }
  }

  getLatestUpdaterApprovedVersion() {
    try {
      return process.env.UPDATE_AVAILABLE_VERSION;
    } catch (error) {
      return false;
    }
  }

  async getOrgProducts() {
    const {
      NRORG,
      LICENSE_URL
    } = process.env;
    let result = { isError: false, response: null };

    try {
      const filter = {
        requestType : 'FilterData',
        filter : [ {
          'name' : 'NRORG', 'operator' : '=', 'value' : NRORG
        } ]
      };
      const axiosRep = await axios.post(`${LICENSE_URL}/getOrgProducts`, filter, { timeout: 60000 });
      
      if(axiosRep.data.dataset && axiosRep.data.dataset.products) {        
        let licenseProductIds = [];
        const licenseProducts = axiosRep.data.dataset.products;
        let updaterProducts = await updaterProdutosModel.getUpdaterProdutos();
        updaterProducts = updaterProducts.rows;
        let { rows } = await adqGetDashboardFiliais.get(paginacao());
        const units = rows.map((unit) => unit.CDFILIAL);
         
        //Insert/Update UPDATERPRODUTOS
        await Promise.all(licenseProducts.map(async (product) => {
          licenseProductIds.push(product.ID.toString());
          let auxProd = updaterProducts.find((item) => item.ID == product.ID);
          if(!auxProd) {
            await this.insertUpdaterProdutos(product);
            await this.insertUpdaterProdVersao(product.VERSIONS, product.ID);
            await this.insertUpdaterProdFili(units, product.ID);
          } else {
            await this.updateUpdaterProdutos(product.ID, product);
            
            const actualVersions = await updaterProdVersaoModel.getUpdaterProdVersao(product.ID);
            const actualVersionIds = actualVersions.rows.map((version) => { return version.NRVERSAO });
            await this.insertUpdaterProdVersao(product.VERSIONS, product.ID, actualVersionIds);

            const actualUnits = await updaterProdFiliModel.getUpdaterProdFili(product.ID);
            const actualUnitIds = actualUnits.rows.map((unit) => { return unit.CDFILIAL });
            await this.insertUpdaterProdFili(units, product.ID, actualUnitIds);
          }
        }));
        
        //Cancel UPDATERPRODUTOS
        await Promise.all(updaterProducts.map(async (updaterProduct) => {
          if(!licenseProductIds.includes(updaterProduct.ID.toString())) {
            await updaterProdutosModel.inactiveUpdaterProdutos(updaterProduct.ID);
          }
        }));

        // Insert UPDATERPRODCONFIG for products without initial config
        await this.handleProductConfig().catch(error => SystemLogger.error(error));
        
        //Download all pending/error versions in background
        const localPackages = {};
        const versionToDownload = await updaterProdVersaoModel.getUpdaterProdVersaoToDownload();
        
        versionToDownload.rows.forEach(async (version) => {
          if (!localPackages[version.DSDIRETORIO]) {
            this.loadLocalPackages(localPackages, version.DSDIRETORIO);
          }

          let prodPackages = localPackages[version.DSDIRETORIO];
          
          if (!prodPackages || !prodPackages.includes(`${version.NRVERSAO}.tar.gz`)) {
            this.downloadVersion(version.DSURL, version.NRVERSAO, version.DSDIRETORIO, version.ID);
          } else if (version.IDDOWNLOADVER !== 'S') {
            await this.updateUpdaterProdVersao(version.ID, 'S');
          }
        });

        await OracleService.commit();
        result.response = licenseProducts;
      } else {
        OracleService.rollback();
        result.isError = true;
        if(axiosRep.data.error) {
          result.response = axiosRep.data.error;
        } else if(axiosRep.data.includes('Stack trace')) {
          result.response = axiosRep.data.split('Stack trace')[0];
        } else {
          result.response = axiosRep.data;
        }
      }
      return result;
    } catch (error) {
      OracleService.rollback();
      result.isError = true;
      if(error.toJSON) {
        error = error.toJSON();
      }
      result.response = error.message;
      return result;
    }
  }
}

module.exports = new ApiController();
