const parametrizacoes = require('../../model/parametrizacoes');
const adqGetDashboardFiliais = require('../../model/adqGetDashboardFiliais');
const apiController = require('../../controller/ApiController');
const updaterService = require('../../service/UpdaterService');
const Gzip = require('../../functions/Gzip');
const SocketService = require('../../service/SocketService');
const Query = require('../../helpers/Query');
const ServicesMonitoring = require('../../helpers/ServicesMonitoring');

const route = (router) => {
  router.get('/data/:resource', async (req, res) => {
    try {
      const { resource } = req.params;
      const { page = 1, limit = 999 } = req.query;
      switch( resource ) {
        case 'monitoring':
          const monitoringParams = await adqGetDashboardFiliais.fetchAssoc();
          const clientsConnected = await SocketService.getMonitoringSocket().getClients();
          const unitsConnected = clientsConnected.map(socket => socket.CDFILIAL );
          if(!monitoringParams) return;
          const { rows: data } = monitoringParams;
          data.forEach( item => {
            item.sat = item.SAT === 0 ? 'N' : 'E';
            item.websocket = unitsConnected.includes(item.CDFILIAL) ? 'S' : 'N';
          });
          return res.send({
            data,
            pagination: {
              limit: 10,
              total: data.length
            }
          });
        case 'orgProducts':
          const productsResult = await apiController.getOrgProducts();
          if(!productsResult) return;
          return res.send(productsResult);
        case 'parametrizacoes':
          const assistantConfig = await parametrizacoes.getAssConfig();
          if (!assistantConfig) return;
          
          for (const index in assistantConfig.rows) {
            assistantConfig.rows[index].COMANDO = Gzip.decode(assistantConfig.rows[index].COMANDO);
          }

          res.send({ 
            data: assistantConfig.rows,
            pagination: {
              page,
              limit,
              total: assistantConfig.rows.length
            }
          });
          break;
        case 'parametrizacoesFilial':
          const units = await parametrizacoes.getFilial();
          if(!units || !units.rows) return;
          let updatersConnected = await SocketService.getUpdaterSocket().getUpdaters();
          const connectedFiliais = updatersConnected.map((socket) => {
            return socket.CDFILIAL;
          });
          res.send({
            data: units.rows.map( unit => {
              unit.CONNECTION_STATUS = connectedFiliais.includes(unit.CDFILIAL);
              return unit;
            }),
            pagination: {
              limit: 999,
              total: units.rows.length
            }
          });
          break
        case 'productsVersion':
          const productsVersion = await updaterService.getProductsVersion(req.query);
          if(productsVersion) {
            res.send({
              data: productsVersion.rows,
              pagination: {
                limit: 999,
                total: productsVersion.rows.length
              }
            });
          }
          break;
        case 'parametrizacoesApacheCloud' :
          const { rows } = await parametrizacoes.getMonitoringParams(true);
          const monitoringResult = SocketService.getMonitoringApacheCloud().getMonitoringResults();
          rows.forEach(row => {
            let monitoring = monitoringResult[row.ID] || [];
            row.STATUS = monitoring.every(item => item.STATUS === 'S') ? 'S' : 'N';
          });
          res.send({
            data: rows,
            pagination: {
              limit: 999,
              total: rows.length
            }
          });
          break;
        default:
          res.status(404).send('ROUTE_NOT_FOUND');
      }
    } catch (error) {
      res.status(500).send({error: true, message: error.message});
    }
  });

  router.post('/data/:resource', async (req, res) => {
    const { resource } = req.params;
    try {
      switch (resource) {
        case 'monitoringConfig':
          
            const { COMANDO } = req.body;

            const COMANDO_ENCODED = Gzip.encode(COMANDO);

            if (COMANDO_ENCODED.length > 4000) {
              throw 'MAX_CHARACTER_REACHED_COMMAND';
            }
            const params = { ...req.body, COMANDO: COMANDO_ENCODED };
            delete params.ID;
            params.COMPLEMENTOTIPO = params.COMPLEMENTOTIPO || 'N';
            const result = await parametrizacoes.insertMonitoringConfig(params);

            result.connection.commit();
            const [RESULT_ID] = result.outBinds.ID;
            if (req.body.ATIVO === 'S') {
              const config = { ...req.body, ID: RESULT_ID };
              if(config.TIPO === 'C') {
                SocketService.getMonitoringApacheCloud().updateMonitoringParams(config);
              } else {
                SocketService.getMonitoringSocket().updateMonitoringParams(config);
              }
            }
            res.send({
              error: false,
              params: {
                ID: RESULT_ID,
              },
              message: 'SUCCESS_INSERT_RECORD',
            });
          break;
        default:
          res.send(404).json({ error: true, message: 'RESOURCE_NOT_FOUND' });
      }
    } catch (error) {
      res.send({
        error: true,
        message: error.message,
      });
    }
  });

  router.put('/data/:resource/:ID', async (req, res) => {
    const { resource } = req.params;
    try {
      switch (resource) {
        case 'monitoringConfig':
          let { COMANDO, TIPO } = req.body;

          if(['L', 'A'].includes(TIPO) && !Query.validate(COMANDO)) {
            throw 'INVALID_COMMAND_QUERY';
          }
          if(TIPO === 'S') {
            COMANDO = ServicesMonitoring.removeMonitoringParamInconsistencies(COMANDO);
          }
          const COMANDO_ENCODED = Gzip.encode(COMANDO);

          if (COMANDO_ENCODED.length > 4000) {
            throw 'MAX_CHARACTER_REACHED_COMMAND';
          }

          const result = await parametrizacoes.updateMonitoringConfig({
            ...req.body,
            COMANDO: COMANDO_ENCODED,
          });
          result.connection.commit();

          const config = { ...req.body };
          if(config.TIPO === 'C') {
            SocketService.getMonitoringApacheCloud().updateMonitoringParams(config);
          } else {
            SocketService.getMonitoringSocket().updateMonitoringParams(config);
          }
          res.send({
            error: false,
            message: 'SUCCESS_UPDATE_RECORD',
          });
          break;
        default:
          res.send(404).json({ error: true, message: 'RESOURCE_NOT_FOUND' });
      }
    } catch (error) {
      res.send({
        error: true,
        message: error.message,
      });
    }
  });

  router.delete('/data/:resource/:ID', async (req, res) => {
    const { resource, ID } = req.params;
    try {
      switch (resource) {
        case 'monitoringConfig':
          const result = await parametrizacoes.deleteMonitoringConfig({ ID });
          result.connection.commit();
          SocketService.getMonitoringSocket().stopMonitoring(ID);
          SocketService.getMonitoringApacheCloud().updateMonitoringParams({ ID });
          res.send({
            error: false,
            message: 'SUCCESS_DELETE_RECORD',
          });
          break;
        default:
          res.status(404).send({ error: true, message: 'RESOURCE_NOT_FOUND' });
      }
    } catch (error) {
      res.status(500).send({
        error: true,
        message: error.message,
      });
    }
  });
}

module.exports = route;